# Introduction

This document generated from DocBook source. Please edit DocBook source
rather than the HTML version.

# Workflow

1.  User requests certificate using web form.

2.  When the web form submits, assuming data is valid, the CSR is
    submitted to the InCommon site and, if the submission does not
    result in an error, a new record is created in the `certificate`
    table.

3.  An e-mail is sent to the approvers list.

4.  One of the approvers approves the request at the InCommon site.

5.  At some point after approval InCommon creates the certificate.

6.  A cron that runs periodically detects that the certificate has been
    created and downloads the certificate and puts it into the database.

7.  An e-mail containing the certificate is sent to the requester.

## Technical Support

Comodo technical support can be reached via e-mail: <cmapi@comodo.com>.

  - What if the same request is sent out twice? Should we cancel this
    request during the approval step?

# Database Design

## `requests`

<table style="width:100%;">
<colgroup>
<col style="width: 22%" />
<col style="width: 44%" />
<col style="width: 11%" />
<col style="width: 22%" />
</colgroup>
<thead>
<tr class="header">
<th>Field</th>
<th>Purpose</th>
<th>Data type</th>
<th>Constraint (if any)</th>
</tr>
</thead>
<tbody>
<tr class="odd">
<td><em>id</em></td>
<td>The master record number. This field serves to uniquely identify the record.</td>
<td>int</td>
<td>PRIMARY KEY</td>
</tr>
<tr class="even">
<td><em>cn</em></td>
<td>The CN (common name part of the distinguished name). This is the same as the server name.</td>
<td>varchar(100)</td>
<td>NOT NULL</td>
</tr>
<tr class="odd">
<td><em>cancelled</em></td>
<td>This request has been manually cancelled by a certificate admin. Requests that have been cancelled should not be allowed to proceed, for example, they should not be submitted to InCommon or be allowed to be downloaded(?).</td>
<td>boolean</td>
<td>NOT NULL, DEFAULT false</td>
</tr>
<tr class="even">
<td><em>declined</em></td>
<td>This request has been declined at the InCommon Certificate Manager.</td>
<td>boolean</td>
<td>NOT NULL, DEFAULT false</td>
</tr>
<tr class="odd">
<td><em>rejected</em></td>
<td>This request was rejected by InCommon for some reason. Rejected entries have order numbers but no certificate would have been issued.</td>
<td>boolean</td>
<td>NOT NULL, DEFAULT false</td>
</tr>
<tr class="even">
<td><em>test</em></td>
<td>If this field is true then this certificate is to be provisioned against the Test InCommon site; otherwise this certificate should be provisioned against the production InCommon site.</td>
<td>boolean</td>
<td>NOT NULL, DEFAULT true</td>
</tr>
<tr class="odd">
<td><em>requester_sunetid</em></td>
<td>The SUNet id of the person who requested this certificate.</td>
<td>varchar(12)</td>
<td>NOT NULL</td>
</tr>
<tr class="even">
<td>requester_fullname</td>
<td>The full <strong>name</strong> of the person who requested this certificate. Example: “Adam Lewenberg”.</td>
<td>varchar(60)</td>
<td>NULL</td>
</tr>
<tr class="odd">
<td><em>requester_email</em></td>
<td>The full <strong>e-mail address</strong> of the person who requested this certificate.</td>
<td>varchar(60)</td>
<td>NOT NULL</td>
</tr>
<tr class="even">
<td>requester_affiliation</td>
<td>The affilliation of the person who requested this certificate. Examples: staff, faculty, etc.</td>
<td>varchar(30)</td>
<td>NOT NULL</td>
</tr>
<tr class="odd">
<td>requester_department</td>
<td>The department of the person who requested this certificate. Examples: “Administrative Services”, “IT Services”.</td>
<td>varchar(90)</td>
<td>NULL</td>
</tr>
<tr class="even">
<td>requester_comments</td>
<td>Comments submitted in request.</td>
<td>text</td>
<td>NULL</td>
</tr>
<tr class="odd">
<td>requester_alternatives</td>
<td>Any subject alternative names submitted in request. Note that the text submitted by the requested is <em>not</em> checked to ensure it is the correct format, so you should assume that the text in this field is merely informative.</td>
<td>text</td>
<td>NULL</td>
</tr>
<tr class="even">
<td><em>server_type_id</em></td>
<td>The type of server this certificate is for. Points to the <code>server_type</code> table.</td>
<td>int</td>
<td>NOT NULL, FOREIGN KEY(server_type)</td>
</tr>
<tr class="odd">
<td><em>contact_email</em></td>
<td>The e-mail address where notices concering this certificate (e.g., expiration warnings) should be sent.</td>
<td>varchar(60)</td>
<td>NOT NULL</td>
</tr>
<tr class="even">
<td><em>csr</em></td>
<td>The CSR (certificate signing request) string in PEM (Base-64) format.</td>
<td>text</td>
<td>NOT NULL</td>
</tr>
<tr class="odd">
<td><em>pk_algorithm</em></td>
<td>The public key algorithm used to generate the key-pair. This can be inferred from the CSR's “Public Key Algorithm” field. Examples: id-ecPublicKey, rsaEncryption</td>
<td>string(30)</td>
<td>NOT NULL</td>
</tr>
<tr class="even">
<td><em>reissue</em></td>
<td>This request is for the re-issue of an already provisioned certificate.</td>
<td>boolean</td>
<td>NOT NULL, DEFAULT false</td>
</tr>
<tr class="odd">
<td><em>duration</em></td>
<td>The number of years requested (1, 2, or 3)</td>
<td>int</td>
<td>NOT NULL, IN {1, 2, 3}</td>
</tr>
<tr class="even">
<td>order_number</td>
<td>The InCommmon order number. (At the Certificate Manager web site this is called the “Self-Enrollment Certificate ID”.</td>
<td>varchar(16)</td>
<td>NULL</td>
</tr>
<tr class="odd">
<td>approved</td>
<td>A boolean that when true indicates that the order has been <em>approved</em> on the InCommon Certificate Manager web site. If false, the request has not been approved.</td>
<td>boolean</td>
<td>NOT NULL, DEFAULT FALSE</td>
</tr>
<tr class="even">
<td>fulfilled</td>
<td>A boolean that when true indicates that the order has been <em>fulfilled</em> on the InCommon Certificate Manager web site. If false, the request has not been fulfilled.</td>
<td>boolean</td>
<td>NOT NULL, DEFAULT FALSE</td>
</tr>
<tr class="odd">
<td>certificate_id</td>
<td>When this request is fulfilled and the certificate is downloaded, we add the certificate to the <code>certificates</code> table. The certificate_id is the primary key of this new row in the <code>certificates</code> table.</td>
<td>int</td>
<td>NULL, FOREIGN KEY(certificates)</td>
</tr>
<tr class="even">
<td><em>approval_contacts</em></td>
<td>Comma-delimited list of one more e-mail addresses to notify when the certificate needs approval. Example: <code>cert_approvers@lists.stanford.edu,adamhl@stanford.edu</code>.</td>
<td>varchar(100)</td>
<td>NOT NULL</td>
</tr>
<tr class="odd">
<td>renewid</td>
<td>The InCommmon renewal ID. Usually 20 characters(?). (We name this field “renewid” rather than “renew_id” to indicate that the field is <em>not</em> a reference to a foreign key.) When InCommon renews a certificate, the new certificate gets a new serial number and new start and end dates; everything else stays the same.</td>
<td>varchar(30)</td>
<td>NULL</td>
</tr>
<tr class="even">
<td>certificate_type_id</td>
<td>The type of certificate. This points at the <code>certificate_type</code> table.</td>
<td>int</td>
<td>NOT NULL, FOREIGN KEY (certificate_type)</td>
</tr>
<tr class="odd">
<td>incommon_cert_number</td>
<td>The InCommon certificate type id number. InCommon has a numerical id as well as a text name for each type of certificate it genereates. These can be listed via the InCommon Web API; see also “SSL Web Service API” document. Examples:</td>
<td>int</td>
<td>NULL</td>
</tr>
<tr class="even">
<td>incommon_cert_name</td>
<td>The InCommon certificate type name. See the field <code>incommon_cert_number</code> above for more information.</td>
<td>varchar(60)</td>
<td>NULL</td>
</tr>
<tr class="odd">
<td>suppress_notifications</td>
<td>Suppress expiration notifications for the certificate associated with this request.</td>
<td>boolean</td>
<td>NOT NULL, DEFAULT 0</td>
</tr>
<tr class="even">
<td>notes</td>
<td>Any additional notes for this record.</td>
<td>text</td>
<td>NULL</td>
</tr>
<tr class="odd">
<td><em>created_at</em></td>
<td>The date and time this record was created. <strong>Remember</strong>: this time is Greenwich Mean Time.</td>
<td>datetime</td>
<td>NOT NULL DEFAULT now()</td>
</tr>
<tr class="even">
<td><em>updated_at</em></td>
<td>The date and time this record was last modified. (Will be the same as <code>creation_time</code> until first modification.) <strong>Remember</strong>: this time is Greenwich Mean Time.</td>
<td>datetime</td>
<td>NOT NULL DEFAULT now()</td>
</tr>
<tr class="odd">
<td><em>request_submitted_at</em></td>
<td>The date and time the CSR was submitted to the InCommon Certificate Manager.</td>
<td>datetime</td>
<td>NULL</td>
</tr>
<tr class="even">
<td>Extra trigger: none yet.</td>
<td></td>
<td></td>
<td></td>
</tr>
</tbody>
</table>

## `server_types`

<table style="width:100%;">
<colgroup>
<col style="width: 22%" />
<col style="width: 44%" />
<col style="width: 11%" />
<col style="width: 22%" />
</colgroup>
<thead>
<tr class="header">
<th>Field</th>
<th>Purpose</th>
<th>Data type</th>
<th>Constraint (if any)</th>
</tr>
</thead>
<tbody>
<tr class="odd">
<td><em>id</em></td>
<td>The master record number. This field serves to uniquely identify the record.</td>
<td>int</td>
<td>PRIMARY KEY</td>
</tr>
<tr class="even">
<td><em>name</em></td>
<td>The name of this server type.</td>
<td>varchar(60)</td>
<td>NOT NULL, UNIQUE</td>
</tr>
<tr class="odd">
<td><em>incommonid</em></td>
<td>The InCommon server type id. See section 2.3.1.2 of the API document for the codes: <a href="???">http://www.incommon.org/cert/repository/InCommon_CM_SSL_Web_Service_API.pdf</a>.</td>
<td>int</td>
<td>NOT NULL, UNIQUE</td>
</tr>
<tr class="even">
<td>Extra trigger: none yet.</td>
<td></td>
<td></td>
<td></td>
</tr>
</tbody>
</table>

## `certificate_types`

<table style="width:100%;">
<colgroup>
<col style="width: 22%" />
<col style="width: 44%" />
<col style="width: 11%" />
<col style="width: 22%" />
</colgroup>
<thead>
<tr class="header">
<th>Field</th>
<th>Purpose</th>
<th>Data type</th>
<th>Constraint (if any)</th>
</tr>
</thead>
<tbody>
<tr class="odd">
<td><em>id</em></td>
<td>The master record number. This field serves to uniquely identify the record.</td>
<td>int</td>
<td>PRIMARY KEY</td>
</tr>
<tr class="even">
<td><em>name</em></td>
<td>The name of this certificate type. Examples: “normal”, “wildcard”.</td>
<td>varchar(60)</td>
<td>NOT NULL, UNIQUE</td>
</tr>
<tr class="odd">
<td><em>description</em></td>
<td>A text description of this certificate type.</td>
<td>text</td>
<td>NOT NULL</td>
</tr>
<tr class="even">
<td>Extra trigger: none yet.</td>
<td></td>
<td></td>
<td></td>
</tr>
</tbody>
</table>

## `certificates`

An X509 certificate should be uniquely determined by the pair (issuer,
serial number); see RFC 5280. In the table below, this corresponds to
the pair (issuer, serial\_number). Note that there may be cases where
this is not true, for example, with self-signed certificates generated
locally.

<table style="width:100%;">
<colgroup>
<col style="width: 22%" />
<col style="width: 44%" />
<col style="width: 11%" />
<col style="width: 22%" />
</colgroup>
<thead>
<tr class="header">
<th>Field</th>
<th>Purpose</th>
<th>Data type</th>
<th>Constraint (if any)</th>
</tr>
</thead>
<tbody>
<tr class="odd">
<td><em>id</em></td>
<td>The master record number. This field serves to uniquely identify the record.</td>
<td>int</td>
<td>PRIMARY KEY</td>
</tr>
<tr class="even">
<td><em>issuer</em></td>
<td>The Issuer DN. Example: “C=SE, O=AddTrust AB, OU=AddTrust External TTP Network, CN=AddTrust External CA Root”</td>
<td>varchar(500)</td>
<td>NOT NULL</td>
</tr>
<tr class="odd">
<td><em>serial_number</em></td>
<td>The serial number of this certificate in upper-case hexadecimal.</td>
<td>varchar(60)</td>
<td>NOT NULL</td>
</tr>
<tr class="even">
<td><em>version</em></td>
<td>The certificate version number. Should be one of 1, 2, or 3.</td>
<td>int</td>
<td>NOT NULL, IN {1, 2, 3}.</td>
</tr>
<tr class="odd">
<td><em>subject</em></td>
<td>The subject DN (i.e., the distinguished name). Should be comma-delimited. Example: “C=US, O=Internet2, OU=InCommon, CN=InCommon Server CA”.</td>
<td>varchar(500)</td>
<td>NOT NULL</td>
</tr>
<tr class="even">
<td>cn</td>
<td>The CN (common name part of the distinguished name). Note that not every certificate has a common name (e.g., some Verisign root certificates).</td>
<td>varchar(100)</td>
<td>NULL</td>
</tr>
<tr class="odd">
<td><em>pem</em></td>
<td>The PEM version of the certificate.</td>
<td>text</td>
<td>NOT NULL</td>
</tr>
<tr class="even">
<td><em>valid_at</em></td>
<td>The date and time (GMT) this certificate is valid (same as “Not Before”). <strong>Remember</strong>: this time is Greenwich Mean Time.</td>
<td>datetime</td>
<td>NOT NULL</td>
</tr>
<tr class="odd">
<td><em>expires_at</em></td>
<td>The date and time (GMT) this certificate expires (same as “Not After”). <strong>Remember</strong>: this time is Greenwich Mean Time.</td>
<td>datetime</td>
<td>NOT NULL</td>
</tr>
<tr class="even">
<td>issued_by_id</td>
<td>The id of this certificate's issuer (if self-signed this field should point to the id field of this record).</td>
<td>int</td>
<td>NULL FOREIGN KEY (certificates)</td>
</tr>
<tr class="odd">
<td><em>key_algorithm</em></td>
<td>The algorithm used to generate the key-pair. Should be one of <code>RSA</code> or <code>DSA</code>.</td>
<td>varchar(15)</td>
<td>NOT NULL, IN {<code>RSA, DSA</code>}.</td>
</tr>
<tr class="even">
<td>key_length</td>
<td>The length of the key used to generate this certificate (currently only works with RSA and DSA). Examples: 1024, 2048.</td>
<td>int</td>
<td>NULL</td>
</tr>
<tr class="odd">
<td>signature_algorithm</td>
<td>A certificate's digital signature is generated by taking the hash of the rest of the certificate and then encrypting the hash using the private key of the signer. The <code>signature_algorithm</code> is the name of the algorithm; currently, the algorithms recognized in this field are md2WithRSAEncryption, md5WithRSAEncryption, sha1WithRSAEncryption, and sha256WithRSAEncryption. See also RFC 3279.</td>
<td>varchar(30)</td>
<td>NULL</td>
</tr>
<tr class="even">
<td>subject_key_identifier</td>
<td>The X509v3 Subject Key Identifier. This should be upper-case hex digits only (or NULL). Example: <code>F598F646D62CB280E6BD0EF75641A4CEAAAF55FC</code>. The subject key identifier is usually derived from the public key. Self-signed certificates often do not have subject key identifiers.</td>
<td>varchar(60)</td>
<td>NULL</td>
</tr>
<tr class="odd">
<td>authority_key_identifier</td>
<td>The X509v3 Authority Key Identifier. This field should match the subject_key_identifier field of the signing certificate. If this certificate is self-signed this field may be NULL. This should be upper-case hex digits only (or NULL). Example: <code>E8D75006EE679ECE3E6B87075D5892D009CE75AA</code>. Self-signed certificates often do not have authority key identifiers.</td>
<td>varchar(60)</td>
<td>NULL</td>
</tr>
<tr class="even">
<td><em>ca</em></td>
<td>If the X509v3 Basic Constraint has “CA: TRUE”, set to 1, otherwise set to 0. Note that version 1 certificates will never have this set even if they are self-signed. Note that not every CA certificate is self-signed and not every self-signed certificate has CA set to true.</td>
<td>boolean</td>
<td>NOT NULL DEFAULT 0</td>
</tr>
<tr class="odd">
<td><em>key_cert_sign</em></td>
<td>If the X509v3 Key Usage list of values contains “keyCertSign”, set this field to 1, otherwise, set to 0.</td>
<td>boolean</td>
<td>NOT NULL DEFAULT 0</td>
</tr>
<tr class="even">
<td>crl_url</td>
<td>The URL for the CRL if this certificate were to be revoked.</td>
<td>varchar(120)</td>
<td>NULL</td>
</tr>
<tr class="odd">
<td>ocsp_uri</td>
<td>The URI for the OCSP service.</td>
<td>varchar(120)</td>
<td>NULL</td>
</tr>
<tr class="even">
<td><em>suppress_notifications</em></td>
<td>If this field is set to TRUE, do NOT send e-mail notifications concerning pending expiration. Note that other notifications will still be sent (e.g., notifications concerning initial provisioning).</td>
<td>boolean</td>
<td>NOT NULL DEFAULT 0</td>
</tr>
<tr class="odd">
<td>contact_email</td>
<td>The e-mail address where notices concerning this certificate (e.g., expiration warnings) should be sent. When the certificate is provisioned via the web form, this entry should be a copy of the field of the same name from the <code>requests</code> table. We allow this entry to be NULL so that it will be clear when the corresponding field from the <code>requests</code> table has been copied over.</td>
<td>varchar(60)</td>
<td>NULL</td>
</tr>
<tr class="even">
<td><em>created_at</em></td>
<td>The date and time this record was created.</td>
<td>datetime</td>
<td>NOT NULL DEFAULT now()</td>
</tr>
<tr class="odd">
<td><em>updated_at</em></td>
<td>The date and time this record was last modified. (Will be the same as <code>creation_time</code> until first modification.)</td>
<td>datetime</td>
<td>NOT NULL DEFAULT now()</td>
</tr>
<tr class="even">
<td>Extra constraint: none yet.</td>
<td></td>
<td></td>
<td></td>
</tr>
<tr class="odd">
<td>Extra trigger: none yet.</td>
<td></td>
<td></td>
<td></td>
</tr>
</tbody>
</table>

## `incommon_statuses`

For every certificate issued by InCommon, there is an associated status.
This status is one of 'issued', 'revoked', 'declined', 'expired',
(others?). Over time a certificate can have one or more of these
statuses. This table records the status information for certificates. If
the InCommon certificate manager is queried and a particular certificate
comes back with a status that does not yet appear in this table for that
certificate, we add a row for that certificate-status pair. This implies
that the pair (certificate\_id, status) is unique.

<table style="width:100%;">
<colgroup>
<col style="width: 22%" />
<col style="width: 44%" />
<col style="width: 11%" />
<col style="width: 22%" />
</colgroup>
<thead>
<tr class="header">
<th>Field</th>
<th>Purpose</th>
<th>Data type</th>
<th>Constraint (if any)</th>
</tr>
</thead>
<tbody>
<tr class="odd">
<td><em>id</em></td>
<td>The master record number. This field serves to uniquely identify the record.</td>
<td>int</td>
<td>PRIMARY KEY</td>
</tr>
<tr class="even">
<td><em>certificate_id</em></td>
<td>The id of the certificate in the certificates table this status entry corresponds to.</td>
<td>int</td>
<td>NOT NULL, FOREIGN KEY(certificate.id)</td>
</tr>
<tr class="odd">
<td><em>status</em></td>
<td>The status.</td>
<td>int</td>
<td>NOT NULL, IN {'approved', 'applied', 'declined', 'expired', 'invalid', 'issued', 'rejected', 'requested', 'revoked', 'unmanaged'}</td>
</tr>
<tr class="even">
<td><em>created_at</em></td>
<td>The date and time this record was created.</td>
<td>datetime</td>
<td>NOT NULL DEFAULT now()</td>
</tr>
<tr class="odd">
<td>status_at</td>
<td>The date and time that InCommon set this status. Note that <em>not</em> every status has this date/time. Currently, the following statuses come with an assoiciated date and time: requested, approved, issued, expired, and revoked</td>
<td>datetime</td>
<td>NULL</td>
</tr>
<tr class="even">
<td>Extra constraint: (certificate_id, status) UNIQUE.</td>
<td></td>
<td></td>
<td></td>
</tr>
<tr class="odd">
<td>Extra trigger: none yet.</td>
<td></td>
<td></td>
<td></td>
</tr>
</tbody>
</table>

## `legacy_expirations`

An X509 certificate is uniquely determined by the pair (certificate's
issuer, serial number). In the table below, this corresponds to the pair
(authority\_key\_identifier, serial\_number). If the authority key
identifier is not present, then the Issuer field can be used in its
place.

<table style="width:100%;">
<colgroup>
<col style="width: 22%" />
<col style="width: 44%" />
<col style="width: 11%" />
<col style="width: 22%" />
</colgroup>
<thead>
<tr class="header">
<th>Field</th>
<th>Purpose</th>
<th>Data type</th>
<th>Constraint (if any)</th>
</tr>
</thead>
<tbody>
<tr class="odd">
<td><em>id</em></td>
<td>The master record number. This field serves to uniquely identify the record.</td>
<td>int</td>
<td>PRIMARY KEY</td>
</tr>
<tr class="even">
<td><em>contact_email</em></td>
<td>The e-mail address contact for this legacy certificate.</td>
<td>varchar(60)</td>
<td>NOT NULL</td>
</tr>
<tr class="odd">
<td><em>expires_at</em></td>
<td>The date this legacy certificate expires. Note that this is a DATE <em>not</em> a DATETIME.</td>
<td>date</td>
<td>NOT NULL</td>
</tr>
<tr class="even">
<td><em>cn</em></td>
<td>The CN (common name) of the certificate. Example: “SMTP-TEST.STANFORD.EDU”.</td>
<td>varchar(60)</td>
<td>NULL NULL</td>
</tr>
<tr class="odd">
<td><em>suppress_notifications</em></td>
<td>If this field is set to TRUE, do NOT send e-mail notifications concerning pending expiration.</td>
<td>boolean</td>
<td>NOT NULL DEFAULT FALSE</td>
</tr>
<tr class="even">
<td><em>created_at</em></td>
<td>The date and time this record was created.</td>
<td>datetime</td>
<td>NOT NULL DEFAULT now()</td>
</tr>
<tr class="odd">
<td><em>updated_at</em></td>
<td>The date and time this record was last modified. (Will be the same as <code>creation_time</code> until first modification.)</td>
<td>datetime</td>
<td>NOT NULL DEFAULT now()</td>
</tr>
<tr class="even">
<td>Extra constraint: none yet.</td>
<td></td>
<td></td>
<td></td>
</tr>
<tr class="odd">
<td>Extra trigger: none yet.</td>
<td></td>
<td></td>
<td></td>
</tr>
</tbody>
</table>

## `sans`

These are Subject Alternative Names. Every certificate should have at
least one (i.e., the common name).

<table style="width:100%;">
<colgroup>
<col style="width: 22%" />
<col style="width: 44%" />
<col style="width: 11%" />
<col style="width: 22%" />
</colgroup>
<thead>
<tr class="header">
<th>Field</th>
<th>Purpose</th>
<th>Data type</th>
<th>Constraint (if any)</th>
</tr>
</thead>
<tbody>
<tr class="odd">
<td><em>id</em></td>
<td>The master record number. This field serves to uniquely identify the record.</td>
<td>int</td>
<td>PRIMARY KEY</td>
</tr>
<tr class="even">
<td><em>certificate_id</em></td>
<td>The certificate that has this subject alternative name.</td>
<td>int</td>
<td>NOT NULL FOREIGN KEY (certificates)</td>
</tr>
<tr class="odd">
<td><em>name</em></td>
<td>The subject alternative name. Examples: “adamhl@stanford.edu”, “italertsme.stanford.edu”.</td>
<td>varchar(60)</td>
<td>NOT NULL</td>
</tr>
<tr class="even">
<td><em>type_id</em></td>
<td>The type of the subject alternative name. Points at the <code>san_types</code> table.</td>
<td>int</td>
<td>NOT NULL</td>
</tr>
<tr class="odd">
<td>Extra trigger: none yet.</td>
<td></td>
<td></td>
<td></td>
</tr>
</tbody>
</table>

## `san_types`

These are Subject Alternative Names types. See also
[http://www.ietf.org/rfc/rfc2459.txt](???) section 4.2.1.7.

<table style="width:100%;">
<colgroup>
<col style="width: 22%" />
<col style="width: 44%" />
<col style="width: 11%" />
<col style="width: 22%" />
</colgroup>
<thead>
<tr class="header">
<th>Field</th>
<th>Purpose</th>
<th>Data type</th>
<th>Constraint (if any)</th>
</tr>
</thead>
<tbody>
<tr class="odd">
<td><em>id</em></td>
<td>The master record number. This field serves to uniquely identify the record.</td>
<td>int</td>
<td>PRIMARY KEY</td>
</tr>
<tr class="even">
<td><em>name</em></td>
<td>The subject alternative name type. Examples: “dNSName”, “iPAddress”.</td>
<td>varchar(30)</td>
<td>NOT NULL, UNIQUE</td>
</tr>
<tr class="odd">
<td><em>created_at</em></td>
<td>The date and time this record was created.</td>
<td>datetime</td>
<td>NOT NULL DEFAULT now()</td>
</tr>
<tr class="even">
<td><em>updated_at</em></td>
<td>The date and time this record was last modified. (Will be the same as <code>creation_time</code> until first modification.)</td>
<td>datetime</td>
<td>NOT NULL DEFAULT now()</td>
</tr>
<tr class="odd">
<td>Extra trigger: none yet.</td>
<td></td>
<td></td>
<td></td>
</tr>
</tbody>
</table>

## `eventlog`

<table style="width:100%;">
<colgroup>
<col style="width: 22%" />
<col style="width: 44%" />
<col style="width: 11%" />
<col style="width: 22%" />
</colgroup>
<thead>
<tr class="header">
<th>Field</th>
<th>Purpose</th>
<th>Data type</th>
<th>Constraint (if any)</th>
</tr>
</thead>
<tbody>
<tr class="odd">
<td><em>id</em></td>
<td>The master record number. This field serves to uniquely identify the record.</td>
<td>int</td>
<td>PRIMARY KEY</td>
</tr>
<tr class="even">
<td>request_id</td>
<td>If this event relates to a specific <code>request</code> record, that requests id.</td>
<td>int</td>
<td>NULL FOREIGN KEY (requests)</td>
</tr>
<tr class="odd">
<td>certificate_id</td>
<td>If this event relates to a specific <code>certificate</code> record, that requests id.</td>
<td>int</td>
<td>NULL FOREIGN KEY (certificates)</td>
</tr>
<tr class="even">
<td><em>message</em></td>
<td>The log message.</td>
<td>text</td>
<td>NOT NULL</td>
</tr>
<tr class="odd">
<td><em>level</em></td>
<td>The log level. The numbers have these meanings: 1: <code>DEBUG</code>, 2: <code>INFO</code>, 3: <code>WARN</code>, 4: <code>ERROR</code>.</td>
<td>int</td>
<td>NOT NULL</td>
</tr>
<tr class="even">
<td><em>created_at</em></td>
<td>The date and time this record was created.</td>
<td>datetime</td>
<td>NOT NULL DEFAULT now()</td>
</tr>
<tr class="odd">
<td>Extra trigger: none yet.</td>
<td></td>
<td></td>
<td></td>
</tr>
</tbody>
</table>

# Database Views

## `valid_certificates`

Show the common name and validity dates for all certificates that have
not expired or been revoked.

### SQL

``` 
SELECT
certificates.id AS id,
certificates.cn AS cn,
certificates.expires_at AS expires_at,
certificates.created_at AS created_at
FROM certificates
WHERE ((certificates.expires_at > now()) AND
       (not(certificates.id in
         (
           SELECT
           incommon_statuses.certificate_id
           FROM incommon_statuses
           WHERE (incommon_statuses.status = 'revoked'))
         )
       ))
        
```

## `valid_certificates_max_expires`

The expiration date for a valid certificate (see `valid_certificates`).
Because there can be more than one valid certificate with the same cn,
we need to have this special view to display the one with the latest
expiration date.

### SQL

``` 
SELECT
valid_certificates.cn AS cn,
max(valid_certificates.expires_at) AS mexpires_at
FROM valid_certificates
GROUP BY  valid_certificates.cn
        
```

## `valid_certificates_latest`

We often would like a list of all valid certificates. However, it
happens that there will be more than one valid certificate in the
database with the same cn. Because of this, we have this view that
contains the id, cn, and expiration date for the latest version of any
certificate. That is, if there is more than one valid certificates with
a given cn, only the one with the latest expiration date is shown.

### SQL

``` 
SELECT
v.id AS id,
v.cn AS cn,
v.expires_at AS expires_at
FROM
(
       valid_certificates_max_expires a
  JOIN valid_certificates v
  ON(((a.mexpires_at = v.expires_at) and (v.cn = a.cn)))
)
        
```
