use Test::More tests => 34 ;
BEGIN { use_ok('Stanford::Certificate') };

use Stanford::Orange::Util qw/ slurp trim / ;

use Data::Dumper ;

# Setup schema and config
require 't/setup_schema.pl' ;
my ($schema, $CONFIG) = get_schema() ;

# Create a certificate object from a certificate PEM file.
my $cert = $schema->resultset('Certificate')->new({});
ok ($cert) ;

my $new_cert ;


my $pem_file1 = './t/incommon_root.pem' ;
my $new_cert1 = $cert->make_cert_from_pem_file($pem_file1) ;
ok ($new_cert1->id() > 0) ;
ok ($new_cert1->is_self_signed()) ;
ok ($new_cert1->issued_by_id() == $new_cert1->id()) ;
ok ($new_cert1->check_signing_chain()) ;
ok ($new_cert1->created_at() =~ m{20\d\d\-\d\d\-\d\d \d\d\:\d\d\:\d\d}) ;
ok ($new_cert1->updated_at() =~ m{20\d\d\-\d\d\-\d\d \d\d\:\d\d\:\d\d}) ;
ok ($new_cert1->key_cert_sign()) ;

# Re-read (should still work)
$new_cert1 = $cert->make_cert_from_pem_file($pem_file1) ;
ok ($new_cert1->id() > 0) ;
ok ($new_cert1->is_self_signed()) ;
ok ($new_cert1->issued_by_id() == $new_cert1->id()) ;

my $pem_file2 = './t/incommon_intermediate.pem' ;
my $new_cert2 = $cert->make_cert_from_pem_file($pem_file2) ;
ok ($new_cert2->id() > 0) ;
ok (!$new_cert2->is_self_signed()) ;
ok ($new_cert2->issued_by_id() == $new_cert1->id()) ;
ok ($new_cert2->check_signing_chain()) ;
ok ($new_cert2->key_cert_sign()) ;

# Look at a certificate with SANs (subject alternative names)
my $pem_file3 = './t/multi_domain.pem' ;
my $new_cert3 = $cert->make_cert_from_pem_file($pem_file3) ;
ok ($new_cert3->id() > 0) ;
ok (!$new_cert3->is_self_signed()) ;
ok (!$new_cert3->key_cert_sign()) ;
ok ($new_cert3->issued_by_id() == $new_cert2->id()) ;
ok ($new_cert3->check_signing_chain()) ;
my @sans = $new_cert3->sans() ;
#warn "number of sans is " . scalar(@sans) ;
#warn "here are the sans" . map {warn $_->to_s() ;} @sans ;

ok (scalar(@sans) == 5) ;
ok ($sans[0]->san_type()->name() eq 'dNSName') ;

my $new_cert3_events = $new_cert3->events_formatted() ;
my $event_target = qr/\[info\] inserted new certificate \(serial number, issuer\) = \(48F8C069A20181BA24FB675C60C9781B, C=US, O=Internet2, OU=InCommon, CN=InCommon Server CA\)/ ;
ok ($new_cert3_events =~ m{$event_target}) ;

# Read in a typical file of certificates as supplied by
# InCommon/Comodo. This file contains the requested certificate plus the
# signing cert chain.


# Check that all three certs saved in the database and that the issuer_id
# field is set correctly.
my $pem_file4 = './t/three_cert_chain.pem' ;
my @new_certs4 = $cert->make_cert_from_pem_file($pem_file4) ;
ok (scalar(@new_certs4) == 3) ;

# Search for certificates by subject.
my @certs_found  ;
@certs_found = $cert->find_by_subject('non-existent subject') ;
ok (scalar(@certs_found) == 0) ;

@certs_found = $cert->find_by_subject('C=US, O=Internet2, OU=InCommon, CN=InCommon Server CA') ;
ok (scalar(@certs_found) == 1) ;

# Add a certificate and then a DIFFERENT certificate with the same issuer
# and serial number. This should fail.
my $pem_ss_older = './t/self-signed-older.pem' ;
my $pem_ss_newer = './t/self-signed-newer.pem' ;

my $ss_cert_older = $cert->make_cert_from_pem_file($pem_ss_older) ;
ok ($ss_cert_older->cn() eq 'self-signed-adamhl.stanford.edu') ;

my $ss_cert_newer = $cert->make_cert_from_pem_file($pem_ss_newer) ;
ok ($ss_cert_newer->id() == $ss_cert_older->id()) ;

# Now try to add with "force"
$ss_cert_newer = $cert->make_cert_from_pem_file($pem_ss_newer, 1) ;
ok ($ss_cert_newer->internal_action()) ;
ok ($ss_cert_newer->internal_action() =~ m{force_inserted}) ;
ok ($ss_cert_newer->id() != $ss_cert_older->id()) ;

## save_pems

my @some_pems = (
  trim(slurp('./t/opendns.com.pem')),
  trim(slurp('./t/mdm-dev1.stanford.edu.pem')),
) ;

my @certs = Stanford::Certificate::X509::save_pems(\@some_pems, $schema) ;
ok (scalar(@certs) == 2) ;
