use Test::More tests => 21 ;
BEGIN { use_ok('Stanford::Certificate') };

use Stanford::Orange::Util qw/ slurp trim / ;

use Data::Dumper ;

# Setup schema and config
require 't/setup_schema.pl' ;
my ($schema, $CONFIG) = get_schema() ;

# Create a certificate object from a certificate PEM file.
my $cert = $schema->resultset('Certificate')->new({});
ok ($cert) ;

my %pem_file_to_is_expired = (
  './t/certreq-sbx.stanford.edu.pem'    => 1,
  './t/godaddy_interm.pem'              => 0,
  './t/incommon_intermediate.pem'       => 1,
  './t/incommon_intermediate_2024.pem'  => 0,
  './t/incommon_root.pem'               => 1,
  './t/usertrust-root-2038.pem'         => 0,
  './t/mdm-dev1.stanford.edu.pem'       => 1,
  './t/mdm-dev2.stanford.edu.pem'       => 1,
  './t/mdm-test.stanford.edu.pem'       => 1,
  './t/multi_domain.pem' => 1,
  './t/one_cert.pem'     => 1,
  './t/opendns.com.pem'  => 1,
  './t/renewed01.pem'    => 1,
  './t/renewed02.pem'    => 1,
  './t/stanford-ms-root-ca.pem'      => 0,
  './t/stanford-win-issuer-ca.pem'   => 1,
  './t/stanford-win-root-ca.pem'     => 0,
  './t/three_cert_chain.pem'         => 1,
  './t/verisign_root3.pem'           => 0,
) ;

foreach my $pem_file (sort keys %pem_file_to_is_expired) {
    my $new_cert = $cert->make_cert_from_pem_file($pem_file) ;
    my $msg ;
    if ($pem_file_to_is_expired{$pem_file}) {
      $msg = "confirmed that cert $pem_file has expired" ;
    } else {
      $msg = "confirmed that cert $pem_file has not expired" ;
    }
    ok (($new_cert->is_expired() == $pem_file_to_is_expired{$pem_file}), $msg) ;
}
