use Test::More ;

BEGIN { use_ok('Stanford::Certificate') };

use Data::Dumper ;
use Stanford::Orange::Util qw/ slurp / ;

# Set the DEB_PACKAGE_BUILD environment variable.
require 't/run_non_build_tests.pl' ;

# Setup schema and config
require 't/setup_schema.pl' ;
my ($schema, $CONFIG) = get_schema() ;

# If $cm_credentials_yml_file does not exist, then we are probably
# building inside of git-buildpackage and want to skip these tests
my $number_of_tests_run ;

SKIP:
{
  if (defined($ENV{'DEB_PACKAGE_BUILD'}) && (defined($ENV{'DEB_PACKAGE_BUILD'})eq '1'))
  {
    skip "skipping some collect tests", 4 ;
  }

  my $cm_credentials_yml_file = $CONFIG->incommon_cm_credentials_yaml_file() ;
  skip "InCommon Certificate Manager credentials not found", 4 if (!(-e $cm_credentials_yml_file)) ;

  my ($username, $password) = Stanford::Certificate::Misc::get_username_password($cm_credentials_yml_file);
  ok ($username) ;
  ok ($password) ;

  ok ($schema) ;

  # Make a request
  my $request = $schema->resultset('Request')->new({});
  my $request_existing = $request->create_from_web_page(
    'requester_sunetid'     => 'adamhl',
    'requester_fullname'    => 'Adam Henry Lewenberg',
    'requester_email'       => 'adamhl@stanford.edu',
    'requester_affiliation' => 'Staff',
    'requester_department'  => 'IT Services',
    'cn'                    => 'adamhl557.stanford.edu',
    'server_type_name'      => 'Apache/ModSSL',
    'contact_email'         => 'adamhl@stanford.edu',
    'duration'              => 3,
    'csr'                   => slurp('./t/tcert-dev.csr'),
    'reissue_boolean'       => 0,
    'approval_contacts'     => 'adamhl@stanford.edu',
  ) ;
  $request_existing->test(1) ; # The certificate was originally provisioned against
                               # the test InCommon service

  # We need to use an already approved and issued certificate.
  my $order_number = '141782' ;
  $request_existing->order_number($order_number) ;
  $request_existing->insert() ;

  my $certificate_id = $request_existing->collect($cm_credentials_yml_file) ;

  ok ($certificate_id > 0) ;

  # Make the certificate object.
  my $cert = $schema->resultset('Certificate')->new({});
  my $found_cert = $cert->find_by_id($certificate_id) ;
  ok ($found_cert) ;
  ok ($found_cert->id() == $certificate_id) ;
  ok ($found_cert->cn() eq $request_existing->cn()) ;

  # Check the signing chain.
  ok ($found_cert->check_signing_chain()) ;

}

done_testing( $number_of_tests_run );
