use Test::More tests => 11;

use Data::Dumper ;
use Stanford::Orange::Util qw / slurp / ;
use Stanford::Certificate::Result::Request ;

# # # # # # # # # # # # # # # # # # # # # # # # # #
my $prefix = 'json' ;
sub myok {
    my ($value, $msg) = @_ ;
    return ok($value, "$prefix: $msg") ;
}
# # # # # # # # # # # # # # # # # # # # # # # # # #

my $request_json = slurp('./t/request1.json') ;
my $field_to_value_href =
    Stanford::Certificate::Result::Request::json_to_request($request_json) ;

my $csr = $field_to_value_href->{'csr'} ;
myok ($csr, 'csr field exists') ;

my @objects = Stanford::PKObj::get_objects_from_string($csr) ;
myok (scalar(@objects) == 1, 'found one object in CSR string') ;

$pkobj_csr = $objects[0] ;
my %csr_field_to_value = Stanford::PKObj::parse_req($csr) ;
my $csr_subject = $csr_field_to_value{'subject'} ;
my $csr_md5     = $pkobj_csr->get_md5() ;
myok ($csr_subject =~ m{mdm-dev2}xsm, 'correct subject in CSR') ;
myok ($csr_md5 =~ m{144e7141a0811aa7536f7f71d8cda3c1}ixsm, 'correct CSR MD5') ;

myok(!($field_to_value_href->{'is_wildcard_san'}), 'is not a wildcard san') ;

myok($field_to_value_href->{'duration'} eq '3', 'correct duration') ;
myok($field_to_value_href->{'comment'} eq 'Please hurry up!',
     'correct comment') ;
myok($field_to_value_href->{'contact_email'} eq 'adamhl@stanford.edu',
     'correct contact_email') ;
myok($field_to_value_href->{'server_type'} eq 'Apache/ModSSL',
     'correct server_type') ;

my @sans = @{ $field_to_value_href->{'sans'} } ;
myok(scalar(@sans) ==  2, 'correct number of sans') ;

############

$request_json = slurp('./t/request2.json') ;
$field_to_value_href =
    Stanford::Certificate::Result::Request::json_to_request($request_json) ;

@sans = @{ $field_to_value_href->{'sans'} } ;
myok(scalar(@sans) ==  0, 'correct number of sans (zero)') ;
