use strict ;
use warnings ;

# Use:
#  perl populate.pl $PLATFORM
#
# where $PLATFORM is one of:
#  - testing (for Perl tests)
#  - sbx (Adam's development environment)
#  - uat (for tools-dev)
#  - production (for production)


##############################################################

# We add '../lib' so that during development we can find the modules, and
# we add 'lib' so that when we build the Debian package the script can
# find the modules.
use lib qw( ../lib lib ) ;

use Stanford::Certificate ;
use Stanford::Certificate::Misc ;

my $connect_yml_file = Stanford::Certificate::Misc::get_connect_yml_file() ;

#use Data::Dumper; print Dumper(%Stanford::Certificate::) ;
my $schema = Stanford::Certificate::attach($connect_yml_file);

my %server_name_to_incommonid =
(
'AOL'=> 1,
'Apache/ModSSL'=> 2,
'Apache-SSL (Ben-SSL, not Stronghold)'=> 3,
'C2Net Stronghold'=> 4,
'Cisco 3000 Series VPN Concentrator'=> 33,
'Citrix'=> 34,
'Cobalt Raq'=> 5,
'Covalent Server Software'=> 6,
'IBM HTTP Server'=> 7,
'IBM Internet Connection Server'=> 8,
'iPlanet'=> 9,
'Java Web Server (Javasoft / Sun)'=> 10,
'Lotus Domino'=> 11,
'Lotus Domino Go!'=> 12,
'Microsoft IIS 1.x to 4.x'=> 13,
'Microsoft IIS 5.x and later'=> 14,
'Netscape Enterprise Server'=> 15,
'Netscape FastTrack'=> 16,
'Novell Web Server'=> 17,
'Oracle'=> 18,
'Quid Pro Quo'=> 19,
'R3 SSL Server'=> 20,
'Raven SSL'=> 21,
'RedHat Linux'=> 22,
'SAP Web Application Server'=> 23,
'Tomcat'=> 24,
'Website Professional'=> 25,
'WebStar 4.x and later'=> 26,
'WebTen (from Tenon)'=> 27,
'Zeus Web Server'=> 28,
'Ensim'=> 29,
'Plesk'=> 30,
'WHM/cPanel'=> 31,
'H-Sphere'=> 32,
'OTHER'=> -1,
) ;

my @san_types =
(
  'otherName',
  'rfc822Name',
  'dNSName',
  'x400Address',
  'directoryName',
  'ediPartyName',
  'uniformResourceIdentifier',
  'iPAddress',
  'registeredID',
) ;


use Date::Format;
my $template = "%Y-%m-%d %T" ;

foreach my $name (keys %server_name_to_incommonid)
{
  print "adding a row for '$name'...";

  my $current_time_string = time2str($template, time());

  my $server_type = $schema->resultset('ServerType')->new({}) ;
  $server_type->name($name) ;
  $server_type->incommonid($server_name_to_incommonid{$name}) ;
  $server_type->created_at($current_time_string) ;
  $server_type->updated_at($current_time_string) ;

  $server_type->insert() ;
  print "done\n";
}

my %certificate_type_to_description = (
  'normal'       => "An x509 certificate with a single hostname as subject",
  'code-signing' => "A code-signing certificate",
  'wildcard'     => "A wildcard certificate",
  'ucc'          => "A unified-communications certificate",
) ;

foreach my $name (keys %certificate_type_to_description)
{
  print "adding a row for '$name'...";

  my $current_time_string = time2str($template, time());

  my $certificate_type = $schema->resultset('CertificateType')->new({}) ;
  $certificate_type->name($name) ;
  $certificate_type->description($certificate_type_to_description{$name}) ;
  $certificate_type->created_at($current_time_string) ;
  $certificate_type->updated_at($current_time_string) ;

  $certificate_type->insert() ;
  print "done\n";
}

foreach my $name (@san_types)
{
  print "adding a row for '$name'...";

  my $current_time_string = time2str($template, time());

  my $san_type = $schema->resultset('SanType')->new({}) ;
  $san_type->name($name) ;
  $san_type->created_at($current_time_string) ;
  $san_type->updated_at($current_time_string) ;

  $san_type->insert() ;
  print "done\n";
}

1;
